define([
// dojo
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/Deferred",
    "dojo/Stateful",
    "dojo/Evented",
    "epi/dependency"],
function(declare, lang, Deferred, Stateful, Evented, dependency){

    return declare([Evented],{
        // summary:
        //      A service for interacting with project activities and activity comments
        // tags:
        //      internal

        // activityStore: [readonly] Store
        //      A REST store for interacting with activities.
        activityStore: null,

        // activityCommentStore: [readonly] Store
        //      A REST store for interacting with activity comments.
        activityCommentStore: null,

        constructor: function(params) {
            declare.safeMixin(this, params);

            this.activityStore = this.activityStore || dependency.resolve("epi.storeregistry").get("epi.cms.activities");
            this.activityCommentStore = this.activityCommentStore || dependency.resolve("epi.storeregistry").get("epi.cms.activities.comments");
        },

        addMessage: function (projectId, contentLink, comment) {
            // summary:
            //     Adds a message activity to a content within a project
            // projectId: Number
            //      The project id to where the activity should belong to
            // contentLink: String
            //      The content link of the content that the activity should belong to
            // comment: String,
            //      The comment
            // tags:
            //      public

            var obj = {
                projectId: projectId,
                contentLink: contentLink,
                message: comment
            };
            return this.activityStore.add(obj).then(lang.hitch(this, "refreshActivities"));
        },

        saveMessage: function (messageActivityId, comment) {
            // summary:
            //     Adds a message activity to a content within a project
            //
            // messageActivityId: Number
            //      The id of the message activity to update
            // comment: String,
            //      The comment
            // tags:
            //      public

            var obj = {
                id: messageActivityId,
                message: comment
            };
            return this.activityStore.put(obj).then(lang.hitch(this, "refreshActivities"));
        },

        addComment: function (activityId, comment) {
            // summary:
            //     Adds a "comment" to an activity
            //
            // activityId: Number
            //      The id of the activity to add a comment for
            // comment: String,
            //      The comment
            // tags:
            //      public

            var obj = {
                activityId: activityId,
                message: comment
            };
            return this.activityCommentStore.add(obj).then(lang.hitch(this, function(comment){
                return this.refreshActivity(comment.activityId);
            }));
        },

        saveComment: function (commentId, activityId, comment) {
            // summary:
            //     Saves an already existing "comment"
            //
            // commentId: Number
            //      The id of the comment to update
            // activityId: Number
            //      The id of the activity the comment belongs to
            // comment: String,
            //      The comment
            // tags:
            //      public

            var obj = {
                id: commentId,
                activityId: activityId, // TODO: Update the server so we do not need to provide the activity id when the id is given
                message: comment
            };

            return this.activityCommentStore.put(obj).then(lang.hitch(this, function(comment){
                return this.refreshActivity(comment.activityId);
            }));
        },

        refreshActivity: function (activityId) {
            // summary:
            //      Refreshes the specific activity in the store
            // activityId: Number
            //      The id of the activity to refresh
            // tags:
            //      public

            return this.activityStore.refresh(activityId);
        },

        refreshActivities: function() {
            // summary:
            //      Refreshes the activity feed.
            // tags:
            //      public

            this.emit("refresh-activities");
        }
    });
});
